// Distributed under the MIT software license, see the accompanying
// file LICENSE or http://www.opensource.org/licenses/mit-license.php.
#ifndef QRLLIB_QRLADDRFORMAT_H
#define QRLLIB_QRLADDRFORMAT_H

enum eAddrFormatType {
    SHA256_2X = 0,
};

#endif
